# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - SectionBreakout

    Break sections out of a page, making new pages for each of the sections and
    replacing them with Include macros.

    @copyright: 2011 Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL, see COPYING for details.
"""

Dependencies = ['pages']

from MoinMoin.action import ActionBase
from MoinMoin.PageEditor import PageEditor
from MoinContentSupport import *
import re

# Action class and supporting functions.

class SectionBreakout(ActionBase, ActionSupport):

    "An action breaking sections out of pages."

    def get_form_html(self, buttons_html):
        _ = self._
        request = self.request
        page = self.page
        form = self.get_form()

        level = int(form.get("level", ["2"])[0])

        # Acquire heading details from the page.

        body = page.get_raw_body()
        heading_details = getHeadingDetails(body, level, level)

        d = {
            "buttons_html"                  : buttons_html,
            "heading_level_label"           : escape(_("Heading level")),
            "found_headings_label"          : escape(_("Headings found in page")),
            "propagate_categories_label"    : escape(_("Propagate categories")),
            "preview_label"                 : escape(_("Preview")),
            "level"                         : escattr(level),
            }

        html = u'''
  <table>
    <tr>
      <td class="label">%(heading_level_label)s</td>
      <td><input type="text" name="level" value="%(level)s" size="2" /></td>
    </tr>
    <tr>
      <td class="label">%(found_headings_label)s</td>
      <td>''' % d

        for heading, level, span in heading_details:
            html += "%s<br />" % heading

        html += '''
      </td>
    </tr>
    <tr>
      <td class="label">%(propagate_categories_label)s</td>
      <td class="buttons"><input type="checkbox" name="propagate" value="yes" /></td>
    </tr>
    <tr>
      <td></td>
      <td class="buttons"><input type="submit" value="%(preview_label)s" />%(buttons_html)s</td>
    </tr>
  </table>
''' % d

        return html

    def do_action(self):

        "Create the new event."

        _ = self._
        form = self.get_form()

        # A heading level must be provided.

        level = form.get("level", [None])[0]
        propagate = form.get("propagate", [None])[0]

        if not level:
            return 0, _("No heading level specified.")

        return self.break_out_headings(int(level), propagate)

    def render_success(self, msg, msgtype=None):

        """
        Render neither 'msg' nor 'msgtype' since redirection should occur
        instead.
        NOTE: msgtype is optional because MoinMoin 1.5.x does not support it.
        """

        pass

    def break_out_headings(self, level, propagate):

        """
        Break out headings at the given 'level' from the current page. If
        'propagate' is a true value, propagate categories to subpages.
        """

        _ = self._
        request = self.request
        page = self.page
        formatter = request.formatter

        # Acquire all heading details from the page.

        page_body = page.get_raw_body()

        if propagate:
            categories = getCategoryMembership(page_body)

        regions = []
        current_region_start = None

        for heading, found_level, (start, end) in getHeadingDetails(page_body):

            # Upon finding a suitable heading, begin a new region to be broken
            # out.

            if current_region_start is None and found_level >= level:
                current_region_start = heading, start

            # Upon finding a higher-level heading, end any open region.

            elif current_region_start is not None and found_level <= level:
                regions.append(current_region_start + (start,))

                # For headings at the requested level, open a new region.

                if found_level == level:
                    current_region_start = heading, start
                else:
                    current_region_start = None

        # End any open region.

        else:
            if current_region_start is not None:
                heading, region_start = current_region_start

                # Prevent any capture of end-of-page category information.

                l = getCategoryDeclarations(page_body)
                if not l:
                    region_end = len(page_body)
                else:
                    declaration, (start, end) = l[-1]
                    region_end = max(region_start, start)

                regions.append((heading, region_start, region_end))

        # Make new pages for each region, rebuilding the current page body.

        retained_regions = []
        retained_region_start = 0
        new_page_names = {}

        for heading, start, end in regions:

            # Combine the page name and the heading to make a subpage.

            new_page_name = "%s/%s" % (page.page_name, heading)

            # Distinguish between pages which use the same heading.

            n = new_page_names.get(new_page_name, 0)
            if n:
                new_page_names[new_page_name] = n + 1
                new_page_name += " (%d)" % (n + 1)
            else:
                new_page_names[new_page_name] = n + 1

            # Open the page for editing.

            new_page = PageEditor(request, new_page_name)
            new_page_body = page_body[start:end]

            if propagate:
                new_page_categories = getCategoryMembership(new_page_body)

                # Add categories if the parent page has any.

                if new_page_categories != categories:
                    new_page_body += makeCategoryDeclaration(categories)

            # Save the new page.

            try:
                new_page.saveText(new_page_body, 0)
            except PageEditor.Unchanged:
                pass

            # Retain the preceding region for the current page.

            retained_regions.append(page_body[retained_region_start:start])

            # Insert Include macros for the broken out text.

            retained_regions.append("<<Include(%s,,editlink)>>\n" % new_page_name)

            # Start a new region to retain.

            retained_region_start = end

        # Retain any remaining text.

        else:
            retained_regions.append(page_body[retained_region_start:])

        # Edit the current page.

        edited_page = PageEditor(request, page.page_name)
        edited_page_body = "".join(retained_regions)

        if propagate:
            edited_page_categories = getCategoryMembership(edited_page_body)

            # Add categories if the parent page should have any, but these were
            # broken out.

            if edited_page_categories != categories:
                edited_page_body += makeCategoryDeclaration(categories)

        # Save the current page.

        edited_page.saveText(edited_page_body, 0)

        # NOTE: Perhaps show a message upon failure.

        request.http_redirect(page.url(request))
        return 1, None

# Action function.

def execute(pagename, request):
    SectionBreakout(pagename, request).render()

# vim: tabstop=4 expandtab shiftwidth=4
